//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	ArrayEvaluator.h:	--- arithmetic with arrays
//
//	DESCRIPTION:
//		A class ArrayEvaluator is defined which performs arithmetic
//		on arrays.
//
//	EXTERNAL DEPENDENCIES:
//		function/class		package				comments
//		----------------------------------------------------------------------
//		ColumnGenerator		SAS:selectlib
//
//	CHANGE HISTORY:
//		UL, 27/05/98: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers (UL), Wed May 27 18:36:58 MET DST 1998
//
//	$Header: /data/xmm02/cvsadmin/soc_repos/sas/packages/selectlib/src/service/ArrayEvaluator.h,v 1.2 2001/08/10 16:14:58 ulammers Exp $
//

#ifndef ARRAY_EVALUATOR_H
#define ARRAY_EVALUATOR_H

#include <string>
#include "ExprColumnGenerator.h"
#include "Dal.h"

class ArrayEvaluator {
public:
	ArrayEvaluator() {}
		//
		//	PURPOSE:
		//		Create a dummy ArrayEvaluator object
		//


	ArrayEvaluator(const std::string &expression);
		//
		//	PURPOSE:
		//		Create an ArrayEvaluator object with an arithmetic expression
		//
		//	ARGUMENTS:
		//		In:
		//			expression:	the arithmetic expression subsequent
		//						doIt() invocations will
		//						use	to create new arrays;
		//						<expression> must adhere to the syntax rules
		//						detailed in User Documentation, otherwise a
		//						fatal error condition is raised


	void doIt(const std::vector<Array *> arrayVec, Array &outArray);
		//
		//	PURPOSE:
		//		Populate an array according to the arithmetic expression
		//		that object was instantiated with; symbol names in that
		//		expression refer to names of arrays, pointer to which
		//		have to be passed to doIt()
		//
		//	ARGUMENTS:
		//		In:
		//			arrayVec:	vector of pointers to arrays referred to in the
		//						expression via symbolic names; array names
		//						are associated to pointers in <arrayVec>
		//						in ascending order, e.g. if the expression:
		//								A1 + sin(A1) - A2 ** X
		//						was given, the vector <arrayVec> must contain
		//						three pointers to arrays corresponding to A1,
		//						A2, and X respectively.	All arrays in the
		//						expression must have the same dimensionality
		//						and extents but not necessarily the same
		//						type. An error condition is raised if arrays
		//						of different dimensionality/extents are used.
		//						The output array can be identical to one of
		//						input arrays.
		//	Out:
		//		OutArray:		the output array to be populated according
		//						to the expression; can be identical to
		//						one of the input arrays in <arraryVec>
		//


	virtual ~ArrayEvaluator() {}
 		//
		//	PURPOSE:
		//		destroy the object constructed with either of the above
		//		constructors

private:
	std::string				_expression;
	ExprColumnGenerator		_columnGenerator;
};

#endif
