//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010, European Space Agency
//
//---------------------------------------------------------------------------
//
//	ColumnGenerator.h:	--- an abstract interface to generate new columns
//
//	DESCRIPTION:
//		A class ColumnGenerator is defined which acts as an abstract interface
//		to create new columns in a table
//
//	CHANGE HISTORY:
//		UL, 1999-04-27: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers (UL), Wed Aug 27 11:48:12 MET DST 1997
//
//	$Header: /data/xmm02/cvsadmin/soc_repos/sas/packages/selectlib/src/service/ColumnGenerator.h,v 1.2 2001/08/10 16:14:58 ulammers Exp $
//

#ifndef COLUMN_GENERATOR_H
#define COLUMN_GENERATOR_H

#include <string>
#include "Evaluator.h"
#if !defined(NON_SAS)
	#include "Dal.h"
#endif

class ColumnGenerator {
public:
	ColumnGenerator(const std::string &columnName = "TEMP",
					Dal::DataType dataType = Dal::Real64)
		: _columnName(columnName), _dataType(dataType) {}

		//
		//	PURPOSE:
		//		Create a ColumnGenerator object
		//
		//	ARGUMENTS:
		//		In:
		//			columnName: the name of the new column to construct
		//			dataType  : the type of the new column to construct


	std::string columnName() const { return _columnName; }
	void setColumnName(const std::string &newColumnName)
		{ _columnName = newColumnName; }
	Dal::DataType dataType() const { return _dataType; }
	void setDataType(Dal::DataType newDataType) { _dataType = newDataType; }
		//
		//	PURPOSE:
		//		Return name/type of the column which this object constructs
		//		or set new values


	virtual void doIt(const Table &inTable, Table &outTable) const = 0;
		//
		//	PURPOSE:
		//		Create a new column in a table potentially using data
		//		from other columns in another table
		//
		//	ARGUMENTS:
		//		In:
		//			inTable:	the table containing the columns the
		//						construction of the new column should
		//						use
		//
		//		InOut:
		//			outTable:	the table to which the newly created column
		//						<colName> will be added; <outTable> and
		//						<inTable> can be identical.

	virtual ~ColumnGenerator() {}
 		//
		//	PURPOSE:
		//		destroy the object constructed with either of the above
		//		constructors

private:
	std::string		_columnName;
	Dal::DataType	_dataType;
};

#endif





