//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	Filename.cc: --- implementation of Filename classe
//
//	DESCRIPTION:
//		Instances of class Filename represent file names
//
//	CHANGE HISTORY:
//		UL, 1998-03-25: 1.0.0: - first version
//		UL, 1998-10-07: 1.1.0: - added implementation of operator<<
//		UL, 1999-01-27: 1.2.0: - minor change to avoid gcc 2.8.1 problem
//
//	AUTHOR:
//		U. Lammers (UL), Wed Mar 25 16:52:23 MET 1998
//
//	$Header: /data/xmm02/cvsadmin/soc_repos/sas/packages/selectlib/src/utils/Filename.cc,v 1.2 2001/08/10 16:15:00 ulammers Exp $
//

#include <cctype>
#include <string>
#include <strstream>
#include "Filename.h"


static std::string blockSpec(const std::string &fileName)
{
	std::string::size_type pos;
	std::string::size_type pos1 = 0;
	if ((pos = fileName.rfind("+")) != std::string::npos)
		return fileName.substr(pos+1, fileName.length()-pos-1);
	else if ((pos = fileName.rfind("[")) != std::string::npos &&
			 (pos1 = fileName.rfind("]")) != std::string::npos)
		return fileName.substr(pos+1, pos1-pos-1);

	return "";
} // blockSpec()


int FileName::blockNr() const
{
	std::string blockName(blockSpec(*this));

	if (blockName.length() == 0)
		return 1;

	if (isdigit(blockName[0])) {
		strstream s;
		s<<blockName<<ends;
		int blockNr;
		s>>blockNr;

		return blockNr;
	}

	return -1;
} // FileName::blockNr()


std::string FileName::blockName() const
{
	return blockSpec(*this);
} // FileName::blockName()


std::string FileName::chopBlockSpec() const
{
	std::string::size_type pos;
	if ((pos = this->rfind("[")) != std::string::npos ||
		(pos = this->rfind("+")) != std::string::npos)
		return this->substr(0, pos);

	return *this;
} // FileName::chopBlockSpec()


std::string FileName::basename() const
{
	std::string::size_type pos;
	if ((pos = this->rfind("/")) != std::string::npos)
		return this->substr(pos+1, this->length()-pos);

	return *this;
} // FileName::basename()


std::ostream &operator<<(std::ostream &os, const FileName &fn)
{ os<<std::string(fn); return os; }
