//--------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	StandardPolygon.cc:	--- implementation part of point-in-polygon test ---
//
//	DESCRIPTION:
//		Routines to test whether given point lies inside a closed polygon
//		(convex or concave) - from "Algorithms in C", Robert Sedgewick, pp. 438
//
//	CHANGE HISTORY:
//		UL, 05/01/98: 1.0.0: - first version
//		UL, 02/04/98: 1.1.0: - bug fix in point-in-polygon test
//		UL, 19/05/98: 1.2.0: - minor changes
//
//	AUTHOR:
//		U. Lammers (UL), Wed Aug 27 11:48:12 MET DST 1997
//
//	$Header: /data/xmm02/cvsadmin/soc_repos/sas/packages/selectlib/src/utils/StandardPolygon.cc,v 1.2 2001/08/10 16:15:01 ulammers Exp $
//

#include "StandardPolygon.h"

namespace StandardPolygon {
inline int ccw(const GeoPoint &p0,
					  const GeoPoint &p1,
					  const GeoPoint &p2)
{
	double dx1, dx2, dy1, dy2;

	dx1 = p1.x - p0.x; dy1 = p1.y - p0.y;
	dx2 = p2.x - p0.x; dy2 = p2.y - p0.y;
	if (dx1*dy2 >= dy1*dx2)						return +1;
	if (dx1*dy2 <= dy1*dx2)						return -1;
	if ((dx1*dx2 <= 0) || (dy1*dy2 <= 0))		return -1;
	if ((dx1*dx1+dy1*dy1) <= (dx2*dx2+dy2*dy2))	return +1;

	return 0;
} // ccw()


inline bool intersect(const GeoLine &l1, const GeoLine &l2)
{
	return ((ccw(l1.p1, l1.p2, l2.p1)
			*ccw(l1.p1, l1.p2, l2.p2)) <= 0)
	  &&   ((ccw(l2.p1, l2.p2, l1.p1)
			*ccw(l2.p1, l2.p2, l1.p2)) <= 0);
} // intersect()


bool insidePolygon(const GeoPoint &t, GeoPoint p[], unsigned N)
{
	unsigned count = 0, j = 0;
	GeoLine lt, lp;

	p[0] = p[N]; p[N+1] = p[1];
	lt.p1 = t; lt.p2 = t; lt.p2.x = 1.e+38;

	for (unsigned i=1; i<=N; ++i) {
		lp.p1 = p[i]; lp.p2 = p[i];
		if (!intersect(lp, lt)) {
			lp.p2 = p[j];
			j = i;
			if (intersect(lp, lt))
				++count;
		}
	}

	return count & 1;
} // insidePolygon()

}









