//
//	complexlibtest: test the selection library with complex expression
//
//	UL, Thu Feb 19 11:21:03 MET 1998
//
//	$Header: /data/xmm02/cvsadmin/soc_repos/sas/packages/selectlib/test/complexlibtest.cc,v 1.7 2001/08/10 16:15:03 ulammers Exp $
//

#include <cassert>
#include <string>
#include <iostream>
#include "Evaluator.h"

int main()
{
  const std::string expr1 = "ABC == 3. && DEFG <= 445. && #ROW==99. &&"
  					   "Rudi > 45. .and. "
  					   "CIRCLE(145.0,379.0,77.0,RAWX,RAWY)";

	Evaluator evaluator(expr1);

	NumericSymbols nidents = evaluator.numericIdentifiers();
	std::cout<<"List of numeric identifiers in expression is:\n";
	for (unsigned n=0; n<nidents.name.size(); ++n)
		std::cout<<"|"<<nidents.name[n]<<"|\n";

	StringSymbols sidents = evaluator.stringIdentifiers();
	std::cout<<"List of string identifiers in expression is:\n";
	for (unsigned n=0; n<sidents.name.size(); ++n)
		std::cout<<"|"<<sidents.name[n]<<"|\n";

	assert(nidents.name.size() == 5);
	assert(nidents.name[4] == "RAWY");
	assert(nidents.name[3] == "RAWX");
	assert(nidents.name[2] == "Rudi");
	assert(nidents.name[1] == "DEFG");
	assert(nidents.name[0] == "ABC");
	double *row = evaluator.rowNumber();
	assert(row);

	*nidents.valuePtr[4] = 379.;
	*nidents.valuePtr[3] = 145.;
	*nidents.valuePtr[2] = 46.;
	*nidents.valuePtr[1] = 444.;
	*nidents.valuePtr[0] = 3.;
	*row = 99.;
	assert(evaluator.evalLogic() == true);	// true

	*nidents.valuePtr[4] = 0.;
	*nidents.valuePtr[3] = 145.;
	*nidents.valuePtr[2] = 46.;
	*nidents.valuePtr[1] = 444.;
	*nidents.valuePtr[0] = 3.;
	assert(evaluator.evalLogic() == false); // false

	//	check GTI filtering
	const string expr2("gti(./gti.ds[1], TIME)");

	Evaluator eval2(expr2);
	NumericSymbols idents2 = eval2.numericIdentifiers();
	double x[] = { 0.			/* out */,
				   7.89205e+07	/* in - first GTI */,
				   7.9e+07		/* out */,
				   7.90772e+07  /* in */,
				   8.32892e+07  /* out */,
				   8.32895e+07	/* in */ };
	bool res[] = { false, true, false, true, false, true };

	for (unsigned i=0; i<sizeof(x)/sizeof(x[0]); ++i) {
		*idents2.valuePtr[0] = x[i];

		assert(eval2.evalLogic() == res[i]);
	}

	std::cout<<"Great - all tests passed\n";
	return 0;
} // main()
