!
!   filtertestf90.f90: test the F90 wrapper for filter class
!
!   UL, Wed Mar 25 13:50:58 MET 1998
!
!   $Header: /data/xmm02/cvsadmin/soc_repos/sas/packages/selectlib/test/filtertestf90.f90,v 1.5 2000/06/22 12:35:01 ulammers Exp $
!
program filtertestf90
    use dal
    use datafilter
    use errorhandling

    type(DataSetT) :: set
    type(TableT)   :: tab
    type(ColumnT)  :: col
    integer(int8), dimension(:), pointer  :: flagData
    integer(int32), dimension(:), pointer :: fromRow, toRow
    integer        :: n

    !
    !   read file `filtertest.ds' created by filtertestcxx
    !
    set = dataSet('filtertest.ds', Modify)
    tab = table(set, 'SPATIAL')

    !
    !   filter the table with a box region - selects everything
    !
    call initFilter('box(128.5,128.5,128,128,0,RAWX,RAWY)')
    if (doSelect(tab, 'FLAG') .ne. numberOfRows(tab)) &
        call fatal('filtertestf90','filter1: wrong number of selected photons')

    col = column(tab, 'FLAG', Read)
    flagData => int8Data(col)
    do n=0, numberOfRows(tab)-1
        if (flagData(n) .eq. 0) &
            call fatal('filtertestf90', 'filter2: wrong selection status')
    enddo

    !
    !   filter the table with a box region - selects nothing
    !
    call initFilter('box(128.5,128.5,0,0,0,RAWX,RAWY)')
    if (doSelect(tab, 'FLAG') .ne. 0) &
        call fatal('filtertestf90','filter3: wrong number of selected photons')

    col = column(tab, 'FLAG', Read)
    flagData => int8Data(col)
    do n=0, numberOfRows(tab)-1
        if (flagData(n) .ne. 0) &
            call fatal('filtertestf90', 'filter4: wrong selection status')
    enddo

    !
    !   filter the table with another region filter - more complex
    !
    call initFilter('((100*sin(RAWX/256*360) > RAWY-200) &&     &
                    &(100*sin(RAWX/256*360) < RAWY-150)) ||     &
                    &((100*sin(RAWY/256*360) > RAWX-200) &&     &
                    &(100*sin(RAWY/256*360) < RAWX-150))')

    if (doSelect(tab, fromRow, toRow) .ne. 19361) &
        call fatal('filtertestf90','filter5: wrong number of selected photons')

    if (size(fromRow) .ne. 501 .or. size(toRow) .ne. 501) &
        call fatal('filtertestf90', &
                   'filter6: we should have 501 blocks of selected rows')

    if (fromRow(0) .ne. 152 .or. toRow(0) .ne. 201 &
        .or. &
        fromRow(39) .ne. 10217 .or. toRow(39) .ne. 10239 &
        .or. &
        fromRow(453) .ne. 60687 .or. toRow(453) .ne. 60714 &
        .or. &
        fromRow(500) .ne. 65429 .or. toRow(500) .ne. 65478) &
        call fatal('filtertestf90', 'filter7: row ranges are wrong')

    call release(set)

    call message(AppMsg, Verbose, 'all tests successfully passed')
end program

