//
//	selectutiltest: test the selector utility library
//
//	UL, Tue Mar 17 16:33:38 MET 1998
//
//	$Header: /data/xmm02/cvsadmin/soc_repos/sas/packages/selectlib/test/selectutiltest.cc,v 1.7 2001/09/26 13:46:41 ulammers Exp $
//

#include <cassert>
#include <string>
#include <iostream>
#include "SelectUtils.h"

int main()
{
	const std::string fixname = "x.ds";
	FileName name = fixname;
	assert(name.blockNr() == 1 && name.chopBlockSpec() == fixname);
	name = fixname + "+23";
	assert(name.blockNr() == 23 && name.chopBlockSpec() == fixname);
	name = fixname + "[999]";
	assert(name.blockNr() == 999 && name.chopBlockSpec() == fixname);

	name = fixname + "[GTI00]";
	assert(name.blockNr() == -1 && name.blockName() == "GTI00" &&
		   name.chopBlockSpec() == fixname);

	FileName x("./xxxx"); assert(x.basename() == "xxxx");
	FileName empty; assert(empty.basename() == "");
	FileName x1("xxxx"); assert(x1.basename() == "xxxx");
	FileName x2("1/1/1/1/122.3.4/xxxx"); assert(x2.basename() == "xxxx");

	Gti gti(100., 200.);
	assert(gti.start() == 100. && gti.stop() == 200.);
	assert(gti.inside(100.) && gti.inside(150.) && !gti.inside(99.9999));

	InOGIPgtiTable gtiSet("gti.ds");
	GtiList l;
	gtiSet>>l;
	const Gti *p;
	assert((p = l.inside(8.32895e+07)) &&
		   p->start() > 8.32892e+07 && p->start() < 8.32894e+07 &&
		   p->stop() > 8.32900e+07 && p->stop() < 8.32902e+07);

	GtiList l2;
	l2.push_back(Gti(4., 5.));
	l2.push_back(Gti(6., 7.));
	l2.push_back(Gti(2., 4.));
	l2.push_back(Gti(1., 3.));
	l2.push_back(Gti(0., 3.));

	GtiList l3 = l2.reduce();
	assert(l3.size() == 2);
	assert(l3[0].start() == 0. && l3[0].stop() == 5.);
	assert(l3[1].start() == 6. && l3[1].stop() == 7.);

	OutOGIPgtiTable outgti("testgti.ds");
	outgti<<l3;

	//
	//	region-spec -> expression conversion
	//
	try {
		ImgdisplayRegionSpecification is1(
			"+circle(198.00,195.00,133)      # source");
		assert(is1.toSelectlibExpression("RAWX", "RAWY") ==
				"(RAWX,RAWY) IN circle(198,195,133)");

		assert(is1.toSelectlibExpression("RAWX", "RAWY", 2., 3.) ==
				"(RAWX,RAWY) IN ellipse(395,583,266,399,0)");

		assert(is1.toSelectlibExpression("RAWX", "RAWY", 2., 3., 1000., 2000.)
				== "(RAWX,RAWY) IN ellipse(1394,2582,266,399,0)");

		ImgdisplayRegionSpecification is2(
			"+point(-21.5,51.5)");
		assert(is2.toSelectlibExpression("RAWX", "RAWY") ==
				"(RAWX,RAWY) IN point(-21.5,51.5)");

		DsplotRegionSpecification ds1(
			 "ON;type above;linestyle 1;linewidth 1;color 1;"
			 "line 1267.49271137, 161.873840445, 926.38483965, "
			 "165.352504638;");
		assert(ds1.toSelectlibExpression("PHA") == "PHA IN [926.385:1267.49]");

		DsplotRegionSpecification ds2(
			"ON;type left;linestyle 1;linewidth 1;color 1;"
			"line 856.413994169, 354.359925788, 862.244897959, "
			"283.627087199;");
		assert(ds2.toSelectlibExpression("PHA") == "PHA<=859.329");

		DsplotRegionSpecification ds3(
			"ON;type right;linestyle 1;linewidth 1;color 1;"
			"line 538.629737609, 361.317254174, 547.376093294, "
			"298.701298701;");
		assert(ds3.toSelectlibExpression("PHA") == "PHA>=543.003");

		DsplotRegionSpecification ds4(
			"type polyi;linestyle 1;linewidth 1.0;color 1;xy 0.31568627451, 1.54435107376;xy 0.604901960784, 1.44631185808;xy 0.572222222222, 1.63538748833;xy 0.32385620915, 1.66806722689;xy 0.302614379085, 1.64939309057;on;");
		assert(ds4.toSelectlibExpression("X", "Y") ==
			"(X,Y) IN polygon(0.315686,1.54435,0.604902,1.44631,0.572222,1.63539,0.323856,1.66807,0.302614,1.64939)");

		DsplotRegionSpecification ds5(
			"type polyo;linestyle 1;linewidth 1.0;color 1;xy 0.31568627451, 1.54435107376;xy 0.604901960784, 1.44631185808;xy 0.572222222222, 1.63538748833;xy 0.32385620915, 1.66806722689;xy 0.302614379085, 1.64939309057;on;");
		assert(ds5.toSelectlibExpression("X", "Y") ==
			"!(X,Y) IN polygon(0.315686,1.54435,0.604902,1.44631,0.572222,1.63539,0.323856,1.66807,0.302614,1.64939)");
	} catch (RegionSpecification::InvalidRegionSpec) {
		return 1;
	}

	std::cout<<"All test successfully passed\n";
	return 0;
}

